from django.contrib import admin
from mptt.admin import MPTTModelAdmin

from .models import JednostkaAdministracyjna


class JednostkaAdministracyjnaAdmin(MPTTModelAdmin):
    pass
admin.site.register(JednostkaAdministracyjna, JednostkaAdministracyjnaAdmin)



from teryt_tree.models import JednostkaAdministracyjna
from django_filters.filters import ModelChoiceFilter
from django.utils.translation import ugettext as _


class AreaFilter(ModelChoiceFilter):
    def __init__(self, *args, **kwargs):
        label = kwargs.pop('label', _("Area"))
        required = kwargs.pop('required', False)
        queryset = kwargs.pop('queryset', JednostkaAdministracyjna.objects.all())
        action = kwargs.pop('method', lambda q, _, v: q.area(v))
        super(ModelChoiceFilter, self).__init__(*args,
                                                label=label,
                                                required=required,
                                                queryset=queryset,
                                                method=action,
                                                **kwargs)


class VoivodeshipFilter(AreaFilter):
    def __init__(self, *args, **kwargs):
        label = kwargs.pop('label', _("Voivodeship"))
        queryset = kwargs.pop('queryset', JednostkaAdministracyjna.objects.voivodeship().all())
        super(VoivodeshipFilter, self).__init__(*args, label=label, queryset=queryset, **kwargs)


class CountyFilter(AreaFilter):
    def __init__(self, *args, **kwargs):
        label = kwargs.pop('label', _("County"))
        queryset = kwargs.pop('queryset', JednostkaAdministracyjna.objects.county().all())
        super(CountyFilter, self).__init__(*args, label=label, queryset=queryset, **kwargs)


class CommunityFilter(AreaFilter):
    def __init__(self, *args, **kwargs):
        label = kwargs.pop('label', _("Community"))
        queryset = kwargs.pop('queryset', JednostkaAdministracyjna.objects.community().all())
        super(CommunityFilter, self).__init__(*args, label=label, queryset=queryset, **kwargs)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('teryt_tree', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='category',
            options={'verbose_name': 'Category', 'verbose_name_plural': 'Categories'},
        ),
        migrations.AlterModelOptions(
            name='jednostkaadministracyjna',
            options={'verbose_name': 'Unit of administrative division', 'verbose_name_plural': 'Units of administrative division'},
        ),
        migrations.AlterField(
            model_name='category',
            name='level',
            field=models.IntegerField(db_index=True, choices=[(1, 'voivodeship'), (2, 'county'), (3, 'community')]),
        ),
        migrations.AlterField(
            model_name='jednostkaadministracyjna',
            name='updated_on',
            field=models.DateField(verbose_name='Updated date'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import autoslug.fields
import mptt.fields


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Category',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=50)),
                ('slug', autoslug.fields.AutoSlugField(populate_from=b'name', editable=False)),
                ('level', models.IntegerField(db_index=True, choices=[(1, 'wojew\xf3dztwo'), (2, 'powiat'), (3, 'gmina')])),
            ],
            options={
                'verbose_name': 'Kategoria',
                'verbose_name_plural': 'Kategorie',
            },
        ),
        migrations.CreateModel(
            name='JednostkaAdministracyjna',
            fields=[
                ('id', models.CharField(max_length=7, serialize=False, primary_key=True)),
                ('name', models.CharField(max_length=36, verbose_name='Nazwa')),
                ('slug', autoslug.fields.AutoSlugField(populate_from=b'name', unique=True, editable=False)),
                ('updated_on', models.DateField(verbose_name='Data aktualizacji')),
                ('active', models.BooleanField(default=False)),
                ('lft', models.PositiveIntegerField(editable=False, db_index=True)),
                ('rght', models.PositiveIntegerField(editable=False, db_index=True)),
                ('tree_id', models.PositiveIntegerField(editable=False, db_index=True)),
                ('level', models.PositiveIntegerField(editable=False, db_index=True)),
                ('category', models.ForeignKey(to='teryt_tree.Category', on_delete=django.db.models.deletion.CASCADE)),
                ('parent', mptt.fields.TreeForeignKey(related_name='children', on_delete=django.db.models.deletion.CASCADE, blank=True, to='teryt_tree.JednostkaAdministracyjna', null=True)),
            ],
            options={
                'verbose_name': 'Jednostka podzia\u0142u terytorialnego',
                'verbose_name_plural': 'Jednostki podzia\u0142u terytorialnego',
            },
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.4 on 2016-12-10 21:15
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('teryt_tree', '0002_auto_20151216_0201'),
    ]

    operations = [
        migrations.AlterField(
            model_name='jednostkaadministracyjna',
            name='name',
            field=models.CharField(max_length=36, verbose_name='Name'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-07-12 23:35
from __future__ import unicode_literals

import autoslug.fields
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('teryt_tree', '0003_auto_20161210_1515'),
    ]

    operations = [
        migrations.CreateModel(
            name='SIMC',
            fields=[
                ('id', models.CharField(max_length=7, primary_key=True, serialize=False)),
                ('name', models.CharField(max_length=100)),
                ('updated_on', models.DateField(verbose_name='Updated date')),
                ('sym_pod', models.ForeignKey(blank=True, on_delete=django.db.models.deletion.CASCADE, to='teryt_tree.SIMC')),
            ],
        ),
        migrations.AlterField(
            model_name='category',
            name='slug',
            field=autoslug.fields.AutoSlugField(editable=False, populate_from='name'),
        ),
        migrations.AlterField(
            model_name='jednostkaadministracyjna',
            name='slug',
            field=autoslug.fields.AutoSlugField(editable=False, populate_from='name', unique=True),
        ),
        migrations.AddField(
            model_name='simc',
            name='terc',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='teryt_tree.JednostkaAdministracyjna'),
        ),
    ]



from __future__ import absolute_import

import factory
from teryt_tree import models


class CategoryFactory(factory.django.DjangoModelFactory):
    level = 1
    name = factory.Sequence(lambda n: 'category-{0}'.format(n))

    class Meta:
        model = models.Category


class JednostkaAdministracyjnaFactory(factory.django.DjangoModelFactory):
    name = factory.Sequence(lambda n: 'jst-{0}'.format(n))
    pk = factory.Sequence(lambda n: n)
    category = factory.SubFactory(CategoryFactory)
    updated_on = '2015-05-12'
    active = True
    rght = 0
    level = 0

    class Meta:
        model = models.JednostkaAdministracyjna


def factory_jst():
    return JednostkaAdministracyjnaFactory()



from __future__ import unicode_literals

from autocomplete_light import shortcuts as autocomplete_light
from django.utils.encoding import force_text

from .models import JednostkaAdministracyjna


class JednostkaAdministracyjnaAutocomplete(autocomplete_light.AutocompleteModelBase):
    search_fields = ['name']
autocomplete_light.register(JednostkaAdministracyjna, JednostkaAdministracyjnaAutocomplete)


class CommunityAutocomplete(JednostkaAdministracyjnaAutocomplete):
    def choice_label(self, choice):
        return "{parent} > {choice}".format(parent=force_text(choice.parent),
                                            choice=force_text(choice))

    def choices_for_request(self):
        self.choices = self.choices.filter(category__level=3).select_related('parent')
        return super(CommunityAutocomplete, self).choices_for_request()

autocomplete_light.register(JednostkaAdministracyjna, CommunityAutocomplete)



# -*- coding: utf-8 -*-
from autoslug import AutoSlugField
try:
    from django.urls import reverse
except ImportError:  # For Django 1.8 (EOL "until April 2018")
    from django.core.urlresolvers import reverse
from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext as _
from model_utils import Choices
from mptt.managers import TreeManager
from mptt.models import MPTTModel, TreeForeignKey


class JednostkaAdministracyjnaManager(TreeManager):

    def with_category(self):
        return self.select_related('category')


class JednostkaAdministracyjnaQuerySet(models.QuerySet):

    def voivodeship(self):
        return self.filter(category__level=1)

    def county(self):
        return self.filter(category__level=2)

    def community(self):
        return self.filter(category__level=3)

    def area(self, jst):
        return self.filter(tree_id=jst.tree_id,
                           lft__range=(jst.lft, jst.rght))


@python_2_unicode_compatible
class Category(models.Model):
    LEVEL = Choices((1, 'voivodeship', _('voivodeship')),
                    (2, 'county', _('county')),
                    (3, 'community', _('community')))
    name = models.CharField(max_length=50)
    slug = AutoSlugField(populate_from='name')
    level = models.IntegerField(choices=LEVEL, db_index=True)

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = _('Category')
        verbose_name_plural = _('Categories')


@python_2_unicode_compatible
class JednostkaAdministracyjna(MPTTModel):
    id = models.CharField(max_length=7, primary_key=True)
    parent = TreeForeignKey('self', null=True, blank=True,
                            related_name='children',
                            on_delete=models.CASCADE)
    name = models.CharField(_('Name'), max_length=36)
    category = models.ForeignKey(Category, on_delete=models.CASCADE)
    slug = AutoSlugField(populate_from='name', unique=True)
    updated_on = models.DateField(verbose_name=_("Updated date"))
    active = models.BooleanField(default=False)
    objects = JednostkaAdministracyjnaManager.from_queryset(JednostkaAdministracyjnaQuerySet)()

    def __str__(self):
        return u'{0}'.format(self.name)

    def get_absolute_url(self):
        return reverse('teryt:details', kwargs={'slug': self.slug})

    class MPTTMeta:
        order_insertion_by = ['name']

    class Meta:
        verbose_name = _('Unit of administrative division')
        verbose_name_plural = _('Units of administrative division')


@python_2_unicode_compatible
class SIMC(models.Model):
    id = models.CharField(max_length=7, primary_key=True)
    sym_pod = models.ForeignKey('self', blank=True, on_delete=models.CASCADE)
    terc = models.ForeignKey(JednostkaAdministracyjna, on_delete=models.CASCADE)
    name = models.CharField(max_length=100)
    updated_on = models.DateField(verbose_name=_("Updated date"))

    def __str__(self):
        return "{}".format(self.name)